package com.tianyi.sync.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.tianyi.sync.mapper.master.CashFlowMapper;
import com.tianyi.sync.mapper.slave.FallSlaveMapper;
import com.tianyi.sync.model.CashFlowModel;
import com.tianyi.sync.service.CashFlowService;
import com.tianyi.sync.utils.DateUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StopWatch;


import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class CashFlowServiceImpl implements CashFlowService {

    private final static Logger logger= LoggerFactory.getLogger(CashFlowServiceImpl.class);

    private final CashFlowMapper cashFlowMapper;

    private final FallSlaveMapper fallSlaveMapper;

    public CashFlowServiceImpl(CashFlowMapper cashFlowMapper, FallSlaveMapper fallSlaveMapper) {
        this.cashFlowMapper = cashFlowMapper;
        this.fallSlaveMapper = fallSlaveMapper;
    }

    @Override
    public Object cashFlow() {
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        String period= DateUtils.getYearAndMonthLast();
        //String period="2024-02";
        String yearm = period.replace("-", "");
        List<String> stringList=fallSlaveMapper.getCodeList().stream().map(v->v.replaceAll("[a-zA-Z]", StringUtils.EMPTY)).collect(Collectors.toList());
        List<CashFlowModel> cashFlowModels=new ArrayList<>();
        stringList.forEach(v->{
            List<CashFlowModel> cashFlowModelList=cashFlowMapper.getCashFlowInfo(period, v);
            for (CashFlowModel model : cashFlowModelList) {
                String accountName = model.getItemname();
                // 判断 accountName 的最后四个字符，根据条件设置 itemType
                if (accountName != null && accountName.length() >= 4) {
                    String cashflowChars = accountName.substring(accountName.length() - 4); // 获取最后四个字符

                    if ("票据结算".equals(cashflowChars)) {
                        model.setCashflow("02");  // 设置为 '02'
                    } else if ("内部结算".equals(cashflowChars)) {
                        model.setCashflow("03");  // 设置为 '03'
                    } else {
                        model.setCashflow("01");  // 默认设置为 '01'
                    }
                }
                model.setPeriod(yearm);
            }
            cashFlowModels.addAll(cashFlowModelList);
        });

        if (!CollectionUtils.isEmpty(cashFlowModels)){
            fallSlaveMapper.deleteCashFlow(yearm);
            fallSlaveMapper.save(cashFlowModels);
        }
        stopWatch.stop();
        logger.info("执行耗时:{}",stopWatch.getTotalTimeMillis());
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("msg","保存成功!");
        return jsonObject;
    }
}
