package com.tianyi.sync.service.impl;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.tianyi.sync.mapper.master.QueryMasterMapper;
import com.tianyi.sync.mapper.slave.FallSlaveMapper;
import com.tianyi.sync.model.DataSyncModel;
import com.tianyi.sync.model.FallDataModel;
import com.tianyi.sync.model.ParmDataModel;
import com.tianyi.sync.service.DataSyncService;
import com.tianyi.sync.utils.DateUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StopWatch;

import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;


@Service
public class DataSyncServiceimpl implements DataSyncService {

    private final static Logger logger = LoggerFactory.getLogger(DataSyncServiceimpl.class);

    private final FallSlaveMapper fallSlaveMapper;

    private final QueryMasterMapper queryMasterMapper;

    public DataSyncServiceimpl(QueryMasterMapper queryMasterMapper, FallSlaveMapper fallSlaveMapper) {
        this.queryMasterMapper = queryMasterMapper;
        this.fallSlaveMapper = fallSlaveMapper;
    }

    @Override
    @DSTransactional
    public Object sync() {
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        JSONObject jsonObject = new JSONObject();
        String lastMonth = DateUtils.getYearAndMonthLast();
        ParmDataModel parmDataModel = new ParmDataModel();
        parmDataModel.setPeriod(lastMonth);
//        parmDataModel.setPeriod("2024-06");
        String period = parmDataModel.getPeriod();
        parmDataModel.setYear(period.substring(0, 4));
        parmDataModel.setMonth(StringUtils.startsWith(period.substring(5, 7), "0") ? StringUtils.substringAfterLast(period.substring(5, 7), "0") : period.substring(5, 7));
        logger.info("参数:{}", JSON.toJSONString(parmDataModel));
        //先清除表内数据
        fallSlaveMapper.delete(parmDataModel);
        //查询需要查数据的账簿code
        List<String> icpCodeList = fallSlaveMapper.getCodeList();
        List<String> codeList = new LinkedList<>();
        icpCodeList.forEach(v -> codeList.add(v.replaceAll("[a-zA-Z]", "")));
        List<String> stringList = new LinkedList<>();
        for (String icpCode : codeList) {
            parmDataModel.setEntity(icpCode + "总账科目");
            //查询164库内数据
            List<DataSyncModel> dataSyncModelList = queryMasterMapper.queryCurrentMonth(parmDataModel);
            if (CollectionUtils.isEmpty(dataSyncModelList)) {
                stringList.add(icpCode);
                logger.info("unexpected event {}:", "IcpCode为" + icpCode + "未查询到数据!");
                continue;
            }
            List<FallDataModel> fallDataModelList = dataSyncModelList.stream().map(v -> {
                FallDataModel fallDataModel = new FallDataModel();
                fallDataModel.setYear(v.getYear());
                fallDataModel.setMonth(v.getMonth());
                fallDataModel.setAccountcode(v.getAccount());
                fallDataModel.setEntitycode(v.getEntity());
                fallDataModel.setOrivalue(v.getMoney());
                fallDataModel.setICPcode(v.getICP());
                fallDataModel.setC1(v.getC1());
                fallDataModel.setC2(v.getC2());
                fallDataModel.setC3(v.getC3());
                fallDataModel.setC4(v.getC4());
                return fallDataModel;
            }).collect(Collectors.toList());
            //数据落到222数据库内
            fallSlaveMapper.addAll(fallDataModelList);
        }
        jsonObject.put("description", String.join(",", stringList) + "未查询到数据!");
        jsonObject.put("msg", "数据同步成功!");
        jsonObject.put("code", 200);
        stopWatch.stop();
        logger.info("执行时间:{}", stopWatch.getLastTaskTimeMillis());
        return jsonObject;
    }


}
