package com.tianyi.sync.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.tianyi.sync.mapper.master.ProfitDataSyncMapper;
import com.tianyi.sync.mapper.slave.ProfitSlaveMapper;
import com.tianyi.sync.model.FreeQueryProfitModel;
import com.tianyi.sync.model.IndexAccountModel;
import com.tianyi.sync.model.ParmDataModel;
import com.tianyi.sync.model.ProfitModel;
import com.tianyi.sync.service.ProfitDataSyncService;
import com.tianyi.sync.utils.DateUtils;
import lombok.SneakyThrows;
import lombok.val;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StopWatch;

import java.lang.reflect.Field;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;


@Service
public class ProfitDataSyncServiceImpl implements ProfitDataSyncService {

    private final static Logger logger = LoggerFactory.getLogger(ProfitDataSyncServiceImpl.class);

    private final ProfitSlaveMapper profitSlaveMapper;

    private final ProfitDataSyncMapper profitDataSyncMapper;

    public ProfitDataSyncServiceImpl(ProfitSlaveMapper profitSlaveMapper, ProfitDataSyncMapper profitDataSyncMapper) {
        this.profitSlaveMapper = profitSlaveMapper;
        this.profitDataSyncMapper = profitDataSyncMapper;
    }

    @Override
    @SneakyThrows
    @DSTransactional
    public Object profitSync() {
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        JSONObject jsonObject = new JSONObject();
        String lastMonth = DateUtils.getYearAndMonthLast();
         //String lastMonth = "2024-09";
        ParmDataModel parmDataModel = new ParmDataModel();
        parmDataModel.setPeriod(lastMonth.replaceAll("-", ""));
        parmDataModel.setYear(lastMonth.substring(0, 4));
        String monthStr = lastMonth.substring(5, 7);
        parmDataModel.setMonth(StringUtils.startsWith(monthStr, "0") ? lastMonth.substring(6, 7) : monthStr);
        logger.info("参数:{}", JSON.toJSONString(parmDataModel));
        //先清除表内数据
        profitSlaveMapper.delete(parmDataModel);
        List<Map<String, Object>> codeList = profitSlaveMapper.getCodeList();
        List<String> stringList = new LinkedList<>();
        String str ="";
        for (Map<String, Object> map : codeList) {
            Object codeNew = map.get("code_new");
            if (!ObjectUtils.isEmpty(codeNew)){
                 str = codeNew.toString();
            }else {
                Object code = map.get("code");
                str = code.toString();
            }
            str = str.replaceAll("[A]", "");
            parmDataModel.setEntity(str);
            ProfitModel profitModel = profitDataSyncMapper.getProfit(parmDataModel);
            if (ObjectUtils.isEmpty(profitModel)) {
                stringList.add(str);
                logger.info("unexpected event {}:", "entityCode为" + str + "未查询到数据!");
                continue;
            }
            Field[] fields = profitModel.getClass().getDeclaredFields();
            List<FreeQueryProfitModel> freeQueryProfitModelList = new LinkedList<>();
            for (Field field : fields) {
                String indexname = field.getName();
                if (StringUtils.startsWith(indexname, "ZB")) {
                    List<IndexAccountModel> indexAccountModelList = profitSlaveMapper.getAccount(indexname);
                    if (CollectionUtils.isEmpty(indexAccountModelList)) {
                        continue;
                    }
                    for (IndexAccountModel indexAccountModel : indexAccountModelList) {
                        FreeQueryProfitModel freeQueryProfitModel = new FreeQueryProfitModel();
                        Field declaredField = profitModel.getClass().getDeclaredField(indexname);
                        declaredField.setAccessible(true);
                        Object o = declaredField.get(profitModel);
                        if (o == null) {
                            continue;
                        } else {
                            freeQueryProfitModel.setMoney(StringUtils.contains((String) o, "E") ? "0" : ((String) o));
                        }
                        freeQueryProfitModel.setIndexname(field.getName());
                        freeQueryProfitModel.setAccountcode(indexAccountModel.getAccount());
                        freeQueryProfitModel.setAccountname(indexAccountModel.getIndexname());
                        Object code = map.get("code");
                        freeQueryProfitModel.setEntity(code.toString().replaceAll("[A]",StringUtils.EMPTY));
                        //freeQueryProfitModel.setEntity(str);
                        freeQueryProfitModel.setYearm(parmDataModel.getPeriod());
                        freeQueryProfitModelList.add(freeQueryProfitModel);
                    }
                }
            }


            if (!CollectionUtils.isEmpty(freeQueryProfitModelList)) {

                profitSlaveMapper.save(freeQueryProfitModelList);
            }

        }
        jsonObject.put("description", String.join(",", stringList) + "未查询到数据!");
        jsonObject.put("msg", "数据同步成功!");
        jsonObject.put("code", 200);
        stopWatch.stop();
        logger.info("执行时间:{}", stopWatch.getLastTaskTimeMillis());
        return jsonObject;
    }
}
